package main;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.serviceloader.ServiceListFactoryBean;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;

import pl.edu.pwr.tkubik.serviceloader.api.LogService;

// WARNING: run maven install on other projects otherwise the dependencies in pom.xml will be not satisfied
// More over - SPI in this example project works only in a standard way (a modular approach does not work).

@ComponentScan({ "ex.impl03" }) // there we have logService implementation
@SpringBootApplication
public class ServiceloaderJavaSpringApplication implements CommandLineRunner {

	@Autowired
	List<LogService> simpleServicesAsBean; // this will be populated by entities of classes found in project

	@Autowired
	@Qualifier("logServiceListFactoryBean")
	ServiceListFactoryBean oSimpleServices; // this will be populated by entities of classes found in classpath

	List<LogService> simpleServicesAsSPI;

	@Bean
	public ServiceListFactoryBean logServiceListFactoryBean() {
		ServiceListFactoryBean serviceListFactoryBean = new ServiceListFactoryBean();
		serviceListFactoryBean.setServiceType(LogService.class);
		return serviceListFactoryBean;
	}

	public static void main(String[] args) {
		SpringApplication.run(ServiceloaderJavaSpringApplication.class, args);
	}

	public void run(final String... strings) throws Exception {

		simpleServicesAsBean.forEach(service -> {
			service.log("message");
		});

		simpleServicesAsSPI = (List<LogService>) oSimpleServices.getObject();
		simpleServicesAsSPI.forEach(service -> {
			service.log("message");
		});

	}

}
